@@ -27,13 +27,13 @@ probval.TVPACK <- function (x, n, df, lower, upper, infin, corr, corrF, delta) {
   
     if (n == 2) {
         cr <- as.double(corr[2,1])
-        res <- .Fortran("bvtlrcall", nu, upp[1], upp[2], cr, val = double(1))
+        res <- .C("C_bvtlr", nu, upp[1], upp[2], cr, val = double(1))
     }
 
     if (n == 3) {
         cr <- c(corr[2,1], corr[3,1], corr[3,2])
         cr <- as.double(cr)
-        res <- .Fortran("tvtlrcall", nu, upp, cr, eps, val = double(1))
+        res <- .C("C_tvtlr", nu, upp, cr, eps, val = double(1))
     }
     error <- ifelse(n == 3, x$eps, NA)
     ret <- list(value = res$val, inform = 0, error = error)
